/*------------------------------------------------------------------------------
  File        :   BaseRunner.cls
  Package     :   OEUnit.Runner
  Description :   Base class for a test runner. Custom runners should inherit 
                  from OEUnit.Runners.OEUnitRunner instead of this class. 
  Author      :   Cameron Wills
  Revisions   :   1.0 - July, 2008
                  Initial Implementation.
------------------------------------------------------------------------------*/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING OEUnit.Runner.*.
USING OEUnit.Reflection.*.
USING OEUnit.Runner.Manipulation.IFilter.
USING OEUnit.Runner.Notification.RunNotifier.
USING Progress.Lang.Object.

CLASS OEUnit.Runner.BaseRunner:

  /*----------------------------------------------------------------------------
    Results from the test run.
  ----------------------------------------------------------------------------*/
  DEFINE PUBLIC PROPERTY Results AS TestClassResult NO-UNDO GET. PROTECTED SET.
  
  /*----------------------------------------------------------------------------
    Notifier to notify run listeners of the events occuring during the test run.
  ----------------------------------------------------------------------------*/
  DEFINE PUBLIC PROPERTY Notifier AS RunNotifier NO-UNDO GET. SET.
  
  /*----------------------------------------------------------------------------
    Filter to ignore test classes and methods that you dont want to run 
  ----------------------------------------------------------------------------*/  
  DEFINE PUBLIC PROPERTY Filter AS IFilter NO-UNDO GET. SET.

  DESTRUCTOR BaseRunner():
    DELETE OBJECT Results NO-ERROR.
    DELETE OBJECT Notifier NO-ERROR.
    DELETE OBJECT Filter NO-ERROR.
  END DESTRUCTOR.

  /*----------------------------------------------------------------------------
    The primary entry point for running a test class. Runs the test and stores 
    the results.
  ----------------------------------------------------------------------------*/
  METHOD PUBLIC VOID RunTest(INPUT test AS Object):
    /* Implemented in Subclasses */
  END METHOD.
  
  /*----------------------------------------------------------------------------
    Returns true if the given test should be run as decided by the Filter.
  ----------------------------------------------------------------------------*/  
  METHOD PROTECTED LOGICAL ShouldRun(INPUT inf AS StatementInfo):
    IF VALID-OBJECT(Filter) THEN   
      RETURN Filter:ShouldRun(inf).
    RETURN TRUE.
  END METHOD.
  
  /*----------------------------------------------------------------------------
    Notifies listeners that the test run is about to start 
  ----------------------------------------------------------------------------*/   
  METHOD PROTECTED VOID NotifyOfRunStart(INPUT inf AS ClassInfo):
    IF VALID-OBJECT(Notifier) THEN notifier:FireTestRunStarted(inf).
  END METHOD.
  
  /*----------------------------------------------------------------------------
    Notifies listeners that the test run has finished 
  ----------------------------------------------------------------------------*/
  METHOD PROTECTED VOID NotifyOfRunFinish(INPUT res AS TestClassResult):
    IF VALID-OBJECT(Notifier) THEN notifier:FireTestRunFinished(res).
  END METHOD.

  /*----------------------------------------------------------------------------
    Notifies listeners that a test class/method was ignored  
  ----------------------------------------------------------------------------*/
  METHOD PROTECTED VOID NotifyOfTestIgnored(INPUT res AS TestResult):
    IF VALID-OBJECT(Notifier) THEN notifier:FireTestIgnored(res).
  END METHOD.

  /*----------------------------------------------------------------------------
    Notifies listeners that a test class/method is about to start 
  ----------------------------------------------------------------------------*/
  METHOD PROTECTED VOID NotifyOfTestStart(INPUT inf AS StatementInfo):
    IF VALID-OBJECT(Notifier) THEN notifier:FireTestStarted(inf).
  END METHOD.
  
  /*----------------------------------------------------------------------------
    Notifies listeners that a test class/method has finished  
  ----------------------------------------------------------------------------*/  
  METHOD PROTECTED VOID NotifyOfTestFinish(INPUT res AS TestResult):
    IF VALID-OBJECT(Notifier) THEN notifier:FireTestFinished(res).
  END METHOD.
  
  
END CLASS.